# LSB Test Execution Framework
# Packages management module
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Packages;
use strict;

use Net::FTP;
use POSIX;

require Exporter;
@Packages::ISA = qw(Exporter);
# Exported symbols
push @Packages::EXPORT, qw(
    &install_package &list_files &print_files_list 
    &guess_platform &package_info &remove_package 
    &prepare_test &guess_package_manager &inquire_for_files_list
    $supported_package_managers @all_platforms @all_lsb_versions
    &packages_info %run_version
);

use Common;

# This is the primary package source.
my $FSG_ftp = "ftp.freestandards.org";
my $FSG_lsb = "ftp://$FSG_ftp/pub/lsb";

# FTP handler
my $ftp;

# Mode: i (installed) - package is installed
#       p (present) - file presents in the local package_dir
#       a (available) - file is available for downloading from the ftp source.
#       b (beta) - file is available for downloading from beta source.

# List of all available files. src = "" means local package_dir. 
# filesize is the size to download, it equals 0 for local files.
my $s_files = {}; # $s_files->{$mode}->{lsb_ver}->{arch}->{package_name}->
                      # -> %file = ( FILENAME=>qm-2.2-2.ia32, 
                                   # PACKAGE=>qm, PKG_VER=>2.2-2, MODE=>[ipab],
                                   # SRC=>src, SIZE=>size_to_dwnld, 
                                   # LSB_VER=>3.1, ARCH=>ia32  );

my %files = (); # $files{package_name.version.arch}->%file

# Information about required files for each certain 
# test and architecture
my $test_info = {};    # $test_info->{lsb_ver}->{arch}->{test_name}->
                           # ->{mode}->{FILES}->{package_name}->%file
                           #        \->{VERSION}

# List of all platform names. 
our @all_platforms = ("amd64", "ia32", "ia64", "ppc32", "ppc64", "s390", "s390x");
#our @all_lsb_versions = ("2.0", "3.0", "3.1", "3.2", "4.0");
our @all_lsb_versions = ("3.0", "3.1");

our $supported_package_managers = "RPM and dpkg(via alien)";

# Expiration time for cached FTP listings.
my $expire_period = 60*60*24;
my $oldest_timestamp = undef;

# Information about version of tests to be run
our %run_version = ();
#------------------------------------------------------------------------

# Helps to determine platform name.
sub guess_platform 
{
    my $name = shift;
    
    if ( $name =~ /^(amd64|x86_64)$/)
        { return "amd64" }
    elsif ( $name =~ /^(ia32|i[3-6]86|x86)$/ )
        { return "ia32" }
    elsif ( $name =~ /^(ia64)$/ )
        { return "ia64" }
    elsif ( $name =~ /^(ppc|ppc32)$/ )
        { return "ppc32" }
    elsif ( $name =~ /^(ppc64)$/ )
        { return "ppc64" }
    elsif ( $name =~ /^(s390)$/ )
        { return "s390" }
    elsif ( $name =~ /^(s390x)$/ )
        { return "s390x" }
    else
        { return "unknown"; }
}
#------------------------------------------------------------------------

# Returns 1 if $platform is backward compatible with the host_platform,
# 0 otherwise.
sub is_backward_compatible_platform {
    my $platform = shift;
    
    if ( $platform eq $host_platform ) {
        return 1;
    }
    
    if ( $host_platform eq "amd64" ) {
        if ( $platform eq "ia32" ) {
            return 1;
        }
    } elsif ( $host_platform eq "ia64" ) {
        if ( $platform eq "ia32" ) {
            return 1;
        }
    } elsif ( $host_platform eq "ppc64" ) {
        if ( $platform eq "ppc32" ) {
            return 1;
        }
    } elsif ( $host_platform eq "s390x" ) {
        if ( $platform eq "s390" ) {
            return 1;
        }
    }
    
    return 0;
}

#------------------------------------------------------------------------

# Determines which package manager to use.
sub guess_package_manager {
    debug_inform("Host OS: $host_os");
    if ( $host_os =~ /(suse|red\W?hat|asianux|mandriva|turbolinux)/i )
    {
        # Native package manager for Suse and Redhat is RPM
        return "RPM";
    }
    elsif ( $host_os =~ /(ubuntu|debian|freespire|linspire|rays.?lx)/i )
    {
        # For Debian and Ubuntu use alien and dpkg.
        return "dpkg";
    }
    else {
        if ( does_shell_know ("dpkg") ) {
            return "dpkg";
        } else {
            return "RPM";
        }
    }
    
    return "unknown";
}

#------------------------------------------------------------------------

# Returns info about a package 
# and empty string if the package isn't installed.
sub package_info {
    my $package_name = shift;
    $package_name =~ s/(^\s+|\s+$)//g; #trim
    
    my $result = "";
    
    if ( $host_package_manager eq "RPM" )
    {
        # Use RPM
        $result = capture("rpm -q $package_name"); 
        chomp $result; debug_inform( "| $result");
        $result =~ s/^$package_name-//; # get the version
    }
    elsif ( $host_package_manager eq "dpkg" )
    {
        # Use 'alien' and 'dpkg'.
        debug_inform( "dpkg -s $package_name");
        $result = `dpkg -s $package_name  2>/dev/null | grep '^\\(Version\\|Status\\):'`;
        chomp $result; debug_inform( "dpkg: $?| $result");
            
        if ( $? != 0 ) {
            # Package in qmtest_libstdcpp-*.rpm can be named 
            # like qmtest-libstdcpp.
            $package_name =~ s/_/-/g;
            debug_inform( "dpkg -s $package_name");
            $result = `dpkg -s $package_name  2>/dev/null | grep '^\\(Version\\|Status\\):'`;
            chomp $result; debug_inform( "dpkg: $?| $result");
        }
        
        if ( $? == 0 ) {
            if ($result !~ m/Status:\s+install\s+ok\s+installed/) {
                $? = 1;
            }
        }
        
        if ( $? == 0 ) {
            # In dpkg installed packages marked with 'i' letter on the 
            # second position.
            if ($result =~ m/Version:\s+([^\n]+)/) {
                $result = $1;
            } else {
                fail("Can't understand the answer of dpkg: $result"); 
            }
        }
    }
    else {
        # Unknown package manager
        fail( "package_info: Unknown package manager."
             ." ($host_package_manager FOR $host_os)"
            ); 
    }
    
    if ( $? != 0 ) {
        $result = "";
    }
    
    $result =~ s/\.lsb\d\d?//; # remove lsb(2|20|3)
    
    return $result;
}
#------------------------------------------------------------------------

# Returns info about packages in a hash
sub packages_info {
    my @package_names = @_;
    
    my %result = (); # {package_name}->package_version
    
    if ( $host_package_manager eq "RPM" )
    {
        # Use RPM
        my $resp = capture( "rpm -q ".(join " ",@package_names)." 2>/dev/null" ); 
        chomp $resp; debug_inform( $resp );
        my @lines = split(/\n/, $resp);
        foreach my $line (@lines) {
            my $package_name = shift @package_names;
            # get the version
            if ( $line =~ m/^$package_name-(.*)$/ ) {
                my $ver = $1;
                $ver =~ s/\.lsb\d\d?//; # remove .lsb(2|20|3)
                $result{$package_name} = $ver;
                debug_inform("$package_name | $ver");
            }
        }
    }
    elsif ( $host_package_manager eq "dpkg" )
    {
        # Use 'alien' and 'dpkg'.
        for ( @package_names ) {
            if (/_/) {
                s/_/-/g;
                push @package_names, $_;
            }
        }
        my $resp = capture( "dpkg -l ".(join " ",@package_names)." 2>/dev/null" );
        chomp $resp; debug_inform( $resp );
        
        my @lines = split(/\n/, $resp);
        foreach my $line (@lines) {
            if ( $line =~ /^\wi\s\s+([^\s]+)\s+([^\s]+)/ ) {
                my $package_name = $1;
                my $package_ver = $2;
                foreach my $findname (@package_names) {
                    last if ( $findname eq $package_name );
                    $_ = $findname; s/_/-/g;
                    if ( $_ eq $package_name ) {
                        $package_name = $findname;
                        last;
                    }
                }
                $package_ver =~ s/\.lsb\d\d?//; # remove .lsb(2|20|3)
                $result{$package_name} = $package_ver;
            }
        }
    }
    else {
        # Unknown package manager
        fail( "package_info: Unknown package manager."
             ." ($host_package_manager FOR $host_os)"
            ); 
    }
    
    return \%result;
}
#------------------------------------------------------------------------

# Installs the specified package file.
sub install_package
{
    my $package_file = shift;
    
    # Check that package_file exists
    unless(-f $package_file) {
        inform ("File doesn't exist: $package_file");
        return 1;
    }
    
    # Install it.
    inform ("Installing: $package_file");
    
    if( $host_package_manager eq "RPM" )
    {
        # Use RPM
        cmd ("rpm -i $package_file"); 
    }
    elsif( $host_package_manager eq "dpkg" )
    {
        # Use 'alien' and 'dpkg'.
        cmd ("alien -c -i -k $package_file"); 
    }
    else {
        fail( "install_package: Unknown package manager."
             ." ($host_package_manager, $host_os)"
        ); 
    }
    
    return 0;
}
#----------------------------------------------------------------------------

# Uninstalls the specified package if it is installed.
sub remove_package
{
    my $package_name = shift;

    # Check whether the package is installed.
    if (!package_info($package_name)) {
        # The package seems to be not installed.
        return 1;
    }
    
    # The package seems to be installed, uninstall it.

    # Failure to uninstall package is (mostly) not fatal;
    # either the original version will be used, or a cleanup will not
    # be complete. Try to continue.
    if( $host_package_manager eq "RPM" )
    {
        # Use RPM
        cmd ("rpm -e --allmatches --nodeps $package_name"); 
    }
    elsif( $host_package_manager eq "dpkg" ) {
        # Use 'alien' and 'dpkg'.
        my $response = capture("dpkg -P $package_name 2>&1"); 
        
        if( $? != 0 || $response =~ /dpkg:/ ) {
            # Package in qmtest_libstdcpp-*.rpm can be named 
            # like qmtest-libstdcpp.
            $package_name =~ s/_/-/g; # qmtest_libstdcpp case
            cmd ("dpkg -P $package_name");
        }
    }
    else {
        fail( "remove_package: Unknown package manager."
             ." ($host_package_manager, $host_os)"
        ); 
    }
    
    return 0;
}
#------------------------------------------------------------------------

sub login_ftp {
    # Open ftp
    debug_inform ("FTP init...");
    $ftp = Net::FTP->new($FSG_ftp, Debug => 0, Passive => 1);
    unless ( $ftp ) { fail("FTP init failed."); }
    
    # Login
    debug_inform ("FTP login...");
    unless ( $ftp->login() ) { 
        fail("FTP login (anonymous) failed."); 
    };
}
#------------------------------------------------------------------------

# Reads cached to a file FTP listing.
sub read_listing_from_file {
    my $filename = shift;
    my @listing = (); #result; 
    
    open FILE, $filename or die "Can't open $filename";
    
    while (<FILE>) {
        chomp;
        if ( /^\s*#/ ) {
            if ( /^#TIMESTAMP:(\d{2})-(\d{2})-(\d{4})\s+(\d{2}):(\d{2})$/ ) {
                # Timestamp helps to determine when the cache expires.
                my $timestamp = mktime(0, $5, $4, $1, $2-1, $3-1900);
                if ($timestamp < $oldest_timestamp) { $oldest_timestamp = $timestamp; }
            }
            elsif ( /^#PWD:(.*)$/ ) {
                my $path = $1;
                # One could check the path here
            }
            else {
                debug_inform("Wrong syntax in file $filename");
            }
        }
        else {
            # It is a regular DIR line
            push (@listing, $_);
        }
    }
    close FILE;
    return \@listing;
}
#------------------------------------------------------------------------

# Returns file listing of a given dir on the ftp server.
sub list_ftp
{
    my $pref = shift; # Prefix ("pub/lsb" actually)
    my $src = shift; # path to the dir, which listing should be taken
    
    my $source = $pref.$src;
    
    my @ftp_files = (); # results: @ftp_files[] -> @($source, $filename, $filesize)
    
    my $listing = ();
    
    # Prepare corresponding local directory
    unless (-d "$package_dir/$src" ) {
        cmd ("mkdir -p \"$package_dir/$src\"");
    }
    
    debug_inform ("> SRC $src");
    
    unless( $list_refresh ) {
        # Read cached listing
        if ( -f $package_dir.$src."/ftp_list" ) {
            $listing = read_listing_from_file ($package_dir.$src."/ftp_list");
        }
    } 
    if(!$listing && $download_packages ) { 
        # Refresh the listing.
        if ("-f \"".$package_dir.$src."/ftp_list\"") {
            cmd("rm -f \"".$package_dir.$src."/ftp_list\"");
        }
        unless ($ftp) { login_ftp(); }
        $listing = $ftp->dir($source);
    }
    
    unless($listing) { 
        debug_inform ("WRONG DIR");
        $oldest_timestamp = 0;
        # Return empty list
        return @ftp_files;
    }
    
    # Save listing
    unless ( -f $package_dir.$src."/ftp_list" ) {
        my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday) = localtime ();
        write_string_as_file ($package_dir.$src."/ftp_list", 
            "#TIMESTAMP:"
            .sprintf ("%02d-%02d-%04d %02d:%02d\n", $mday, ($mon + 1), 
            ($year + 1900), $hour, $min )
           ."#PWD:".$source."\n"
           .(join "\n", @$listing)
           );
    }
    
    unless(@$listing) { 
            debug_inform ("< Empty");
            # Return empty list
            return @ftp_files;
    }

    # Parse ftp files listing
    foreach my $line (@$listing) {
        # -rwxr-xr-x 1 user group size date time filename.ext
        if($line =~ m/^([^ ]+)\s+[^ ]+\s+[^ ]+\s+[^ ]+\s+([^ ]+)\s+[^ ]+\s+[^ ]+\s+[^ ]+\s+(.+)$/) {
            my $attrs = $1;
            my $filesize = $2;
            my $filename = $3;
            
            # If it is an ordinary file
            if($attrs =~ /^-/ ) {
                my @line = ($source, $filename, $filesize);
                push @ftp_files, \@line;
            }
        }
        else { 
            debug_inform("Wrong syntax in ftp listing: $line");
        }
    }
    debug_inform ("< Ok");

    return @ftp_files;
}
#----------------------------------------------------------------------------

# Adds file to list of all available files
sub add_file_to_list {
    my ($filename, $package_name, $pkg_ver, $mode, $src, $size, $lsb_ver, $arch) = @_;
    
    # Extract package_name, version and architecture from the filename
    if ( $filename =~ m/^(([^\d][\w_]*-)*)(\d.*)\.(\w+)\.rpm$/ ) {
        my $new_package_name = $1;
        my $new_package_ver = $3;
        my $new_arch = $4;
        if ( !$package_name ) { 
            $package_name = $new_package_name; 
            $package_name =~ s/-$//g;
            
            # tricky case with the lsb-tet3-lite beta for LSB 3.0
            if ($package_name =~ /^lsb-tet3$/ ) { $package_name = "lsb-tet3-lite"; }
        }
        if ( !$pkg_ver ) { $pkg_ver = $new_package_ver; }
        if ( !$arch ) { $arch = guess_platform($new_arch); }
    } else {
        if( $filename =~ m/\.rpm$/ ) {
            debug_inform ( "Bad file name: $filename." );
        }
        if ( !$package_name ) { $package_name = $filename; }
    }

    $pkg_ver =~ s/\.lsb\d\d?$//; # remove .lsb(2|20|3)
    
    foreach my $lsb_ver_i ( @all_lsb_versions ) {
        if ( $lsb_ver && ($lsb_ver ne "all") && ($lsb_ver ne $lsb_ver_i) ) {next;}
        foreach my $arch_i ( @all_platforms ) {
            if ( $arch && ($arch ne "all") && ($arch ne $arch_i) ) {next;}

            my $file  = { FILENAME => $filename, 
                          PACKAGE  => $package_name,
                          PKG_VER  => $pkg_ver,
                          MODE     => $mode,
                          SRC      => $src,
                          SIZE     => $size,
                          LSB_VER  => $lsb_ver_i,
                          ARCH     => $arch_i,
                        };
            if ( (!defined $s_files->{$mode}->{$lsb_ver_i}->{$arch_i}->{$package_name})
              || !($s_files->{$mode}->{$lsb_ver_i}->{$arch_i}->{$package_name}->{PKG_VER} gt $file->{PKG_VER})
            ){
                $s_files->{$mode}->{$lsb_ver_i}->{$arch_i}->{$package_name} = $file;
            } 
            
            if ( !exists($files{"$package_name.$pkg_ver.$arch_i"}) ) {
                $files{"$package_name.$pkg_ver.$arch_i"} = $file;
            }
        }
    }
}
#----------------------------------------------------------------------------

# Gather a set of sources from all tests.
sub gather_sources
{
    my $sources = {}; # Resulting set
    
    foreach my $test_obj (values %tests) {
        # release sources
        foreach my $src ( split " ", $test_obj->sources) {
            next unless $src;
            
            if ($src eq "runtime") {
                foreach my $lsb_ver ( @all_lsb_versions ) {
                    my $runtime_url = "/test_suites/released-"
                                      .$lsb_ver."/binary/runtime";
                    $sources->{"r"}->{$lsb_ver}->{"all"}->{$runtime_url} = 1;
                }
            }
            elsif ($src eq "appbat") {
                foreach my $lsb_ver ( @all_lsb_versions ) {
                    foreach my $arch ( @all_platforms ) {
                        next unless is_backward_compatible_platform($arch);
                        my $appbat_url = "/app-battery/released-".$lsb_ver."/".$arch;
                        $sources->{"r"}->{$lsb_ver}->{$arch}->{$appbat_url} = 1;
                    }
                }
            }
            elsif ($src eq "test_data") {
                my $data_url = "/app-battery/tests";
                $sources->{"r"}->{"all"}->{"all"}->{$data_url} = 1;
            }
            else {
                $sources->{"r"}->{"all"}->{"all"}->{$src} = 1;
            }
        }
        # mode "b"
        foreach my $src ( split " ", $test_obj->beta_sources) {
            next unless $src;
            
            if ($src eq "runtime") {
                my $beta_runtime_url = "/test_suites/beta/binary/runtime";
                $sources->{"b"}->{"all"}->{"all"}->{$beta_runtime_url} = 1;
            }
            elsif ($src eq "appbat") {
                foreach my $arch ( @all_platforms ) {
                    next unless is_backward_compatible_platform($arch);
                    my $beta_appbat_url = "/app-battery/beta/".$arch;
                    $sources->{"b"}->{"all"}->{$arch}->{$beta_appbat_url} = 1;
                }
            }
            elsif ($src eq "test_data") {
                my $data_url = "/app-battery/tests";
                $sources->{"r"}->{"all"}->{"all"}->{$data_url} = 1;
            }
            else {
                $sources->{"b"}->{"all"}->{"all"}->{$src} = 1;
            }
        }
    } # end of foreach my $test_obj
    return $sources;
}
#----------------------------------------------------------------------------

# Generates a list of all available files for each certain platform and mode
sub list_files {
    my $max_mode = shift;
        
    ## Register installed packages
    # Collect all required packages
    my %tmp_hash = ();
    foreach my $test_obj ( values %tests ) {
         foreach my $package_name (split " ", $test_obj->packages) {
             $tmp_hash{$package_name} = 1;
         }
    }
    
    my $result = packages_info (keys %tmp_hash);
    
    foreach my $package_name (keys %$result) {
        my $info = $result->{$package_name};
        if ( $info ) {
            # add installed packages to list
            add_file_to_list("$package_name-$info.$host_platform.rpm",
                $package_name, $info, "p", "", 0, "all", $host_platform );
        }
    }
    
    #----------
    
    ## Register packages in package_dir
    
    my $sources = gather_sources();
    
    foreach my $rb ("r","b") {
        my $mode = "p";
        if ($rb eq "b") { $mode = "b"; }
        foreach my $lsb_ver ( keys %{$sources->{$rb}} ) {
            foreach my $arch ( keys %{$sources->{$rb}->{$lsb_ver}} ) {
                foreach my $src ( keys %{$sources->{$rb}->{$lsb_ver}->{$arch}} ) {
                    my $dir = $package_dir.$src;
                    unless (opendir DIR, $dir) { next; }
                    
                    # Read directory for files and dirs
                    my @dir_list = readdir(DIR);
                    
                    foreach my $filename(@dir_list) {
                        # skip hidden files, . and ..
                        next if $filename =~ /^\./;
                        
                        # '-f' checks it is a file
                        if (-f "$dir/$filename") {
                            add_file_to_list($filename, "","",$mode, $src, 0, $lsb_ver, "");
                        }
                    }
                }
            }
        }
    }
    
    #----------
    
    ## Look for packages on the FTP
    if ( $max_mode =~ /[ab]/ ) 
    {
        # Request FTP files list
        
        # Count up sources
        my $number = 0;
        foreach my $rb (keys %$sources) {
            if ($rb eq "b") { next if ($max_mode ne "b"); }
            foreach my $lsb_ver ( keys %{$sources->{$rb}} ) {
                foreach my $arch ( keys %{$sources->{$rb}->{$lsb_ver}} ) {
                    $number += keys %{$sources->{$rb}->{$lsb_ver}->{$arch}};
                }
            }
        }
        
        $oldest_timestamp = time();
        
        # List dirs
        my $count = 0;
        foreach my $rb (keys %$sources) {
            my $mode = "a";
            if ($rb eq "b") { $mode = "b"; next if ($max_mode ne "b"); }
            
            foreach my $lsb_ver ( keys %{$sources->{$rb}} ) {
                foreach my $arch ( keys %{$sources->{$rb}->{$lsb_ver}} ) {
                    foreach my $src ( keys %{$sources->{$rb}->{$lsb_ver}->{$arch}} ) {
                        
                        my @list = list_ftp("pub/lsb", $src);
                        $count++;
                        if ( $list_refresh || ($verbose_lvl >= 2) ) {
                            print "#PROGRESS: $count/$number\n";
                            print "#\x08 ", "\033[2A","\n";
                        }
                        
                        foreach my $line ( @list ) {
                            my ($source, $filename, $filesize) = @$line;
                            add_file_to_list( $filename,"","",$mode, $src,
                                              $filesize, $lsb_ver, "" );
                        }
                    }
                }
            }
        }
        
        debug_inform ("Done          ");
        
        # Close FTP connction
        if ( $ftp ) {
            $ftp->quit;
            $ftp = undef;
        }
    }
    
    #----------
    
    # Choose right versions of required files for each test
    foreach my $lsb_ver ( @all_lsb_versions ) {
        foreach my $arch ( @all_platforms ) {
            next unless is_backward_compatible_platform($arch);
            foreach my $test_obj (values %tests) {
                my $test_name = $test_obj->name;
                foreach my $mode ("p","a","b") {
                    my @files_to_search = split " ", $test_obj->files;
                    my @packages_to_search = split " ", $test_obj->packages;
#                    my $max_mode = "p";
                    foreach my $filename_search (@files_to_search, @packages_to_search) {
                        my $file = $s_files->{$mode}->{$lsb_ver}->{$arch}->{$filename_search};
                        $max_mode = $mode;
                        if ( ($mode eq "b") && !$file) {
                            $file = $s_files->{"a"}->{$lsb_ver}->{$arch}->{$filename_search};
                            $max_mode = "a";
                        }
                        unless ($file) {
                            $file = $s_files->{"p"}->{$lsb_ver}->{$arch}->{$filename_search};
                            $max_mode = "p";
                        }
                        unless ($file) {
                            # Can't find any suitable file, so this test will be unavailable
                            $test_info->{$lsb_ver}->{$arch}->{$test_name}->{$mode} = undef;
                            #debug_inform("- $mode $lsb_ver $arch $test_name : $filename_search");
                            last;
                        }
#                        if ( ($file->{MODE} eq "a") && ($max_mode eq "p") ) {
#                            $max_mode = "a";
#                        }
#                        elsif ($file->{MODE} eq "b") {
#                            #debug_inform(join " ", "~", $file->{FILENAME}, $file->{MODE}, $file->{ARCH}, $file->{LSB_VER});
#                            $max_mode = "b";
#                        }

                        # select best file
                        $file = $files{$file->{PACKAGE}.".".$file->{PKG_VER}.".".$file->{ARCH}};
                        
                        $test_info->{$lsb_ver}->{$arch}->{$test_name}->{$mode}->{FILES}->{$filename_search}=$file;
                        $test_info->{$lsb_ver}->{$arch}->{$test_name}->{$mode}->{VERSION}=$file->{PKG_VER};
                    }
                    unless ($mode eq $max_mode) {
                        # This mode isn't presented.
                        $test_info->{$lsb_ver}->{$arch}->{$test_name}->{$mode} = undef;
                        #debug_inform("- $mode!=$max_mode $lsb_ver $arch $test_name");
                    }
                }
            }
        }
    }
}
#----------------------------------------------------------------------------

# Produce the text listing of required files
sub print_files_list {
    
    print "# This is a file with the list of all available test modules.
# Syntax:
# ARCHITECTURE: <arch>
#     Current machine architecture
# FILE_SIZE: <file> <size>
#     Size in bytes needed to be downloaded for the file specified
# MODULE_TIME: <module> <time>
#     Time in special units (seconds for a reference machine) needed for the
#     specified test suite to run. These units are then translated into seconds.
# <lsb-version> <arch> <module> <location> <version> <files>
#     <lsb-version> : LSB version for which the information is given
#     <arch>        : machine architecture that the module is for 
#                     (i386, ia64 etc.)
#     <module>      : module name
#     <location>    : location of the test suites ('p' (present) stands for 
#                     localhost, 'a' (available) for FTP server, 'b' (beta) 
#                     for 'beta' subdirectory on the FTP)
#     <version>     : version of the test suite
#     <files>       : comma-separated list of files needed for the test suite
";
    print "ARCHITECTURE: $host_platform\n";
    print "LSB_VERSION: $lsb_version\n";
    
    if (  (defined $oldest_timestamp) && ( $oldest_timestamp < time() )
        && ( abs($oldest_timestamp - time()) > $expire_period )  
        ) {
            if ( $oldest_timestamp > 0 ) {
                my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday) = localtime ($oldest_timestamp + $expire_period);
                print "EXPIRED: ".sprintf ("%02d-%02d-%04d %02d:%02d\n", $mday, ($mon + 1), 
                    ($year + 1900), $hour, $min )."\n";
            } else {
                print "EXPIRED: undef\n";
            }
    }
    
    my %tmp_hash = ();
    foreach my $lsb_ver ( keys %$test_info ) { 
        foreach my $platform ( keys %{$test_info->{$lsb_ver}} ) { 
            foreach my $test_name ( keys %{$test_info->{$lsb_ver}->{$platform}} ) {
                my $tmp = $test_info->{$lsb_ver}->{$platform}->{$test_name};
                foreach my $mode (keys %$tmp)
                {
                    next unless ($tmp->{$mode});
                    
                    foreach my $file (values %{$tmp->{$mode}->{FILES}}) {
                        my $filename = $file->{FILENAME};
                        $tmp_hash{$filename} = $file;
                    }
                }
            }
        }
    }
    foreach my $filename (keys %tmp_hash) {
        my $file = $tmp_hash{$filename};
        print "FILE_SIZE: ".$file->{FILENAME}."\t".$file->{SIZE}."\n";#"\t".$file->{SRC}."\n";
    }
    
    # Print required files for each test, platform and mode
    foreach my $lsb_ver ( keys %$test_info ) { 
        foreach my $platform (keys %{$test_info->{$lsb_ver}}) {
            foreach my $test_name ( keys %{$test_info->{$lsb_ver}->{$platform}} )
            {
                my $tmp = $test_info->{$lsb_ver}->{$platform}->{$test_name};
                foreach my $mode (keys %$tmp)
                {
                    next unless ($tmp->{$mode});
                    print "$lsb_ver\t$platform\t$test_name";
                    my $version = $tmp->{$mode}->{VERSION};
                    print "\t$mode $version";
                    my (@tmp1, @tmp2, @tmp3);
                    foreach my $package_name ( keys %{$tmp->{$mode}->{FILES}} ) {
                        push @tmp1, $tmp->{$mode}->{FILES}->{$package_name}->{PKG_VER};
                        if ( $tmp->{"p"} && $tmp->{"p"}->{VERSION} ) {
                            push @tmp2, $tmp->{"p"}->{FILES}->{$package_name}->{PKG_VER};
                        }
                        if ( $tmp->{"a"} && $tmp->{"a"}->{VERSION} ) {
                            push @tmp3, $tmp->{"a"}->{FILES}->{$package_name}->{PKG_VER};
                        }
                    }
                    if( $mode ne "p" && 
                        (
                            $tmp->{"p"} && @tmp2
                            && ( $tmp->{$mode}->{VERSION} eq $tmp->{"p"}->{VERSION} )
                            && ( "@tmp1" ne "@tmp2" )
                                ||
                            $tmp->{"a"} && @tmp3
                            && ( $tmp->{$mode}->{VERSION} eq $tmp->{"a"}->{VERSION} )
                            && ( "@tmp1" ne "@tmp3" )
                        )
                       )
                       { print "+"; }
                    @tmp1 = ();
                    foreach my $package_name ( keys %{$tmp->{$mode}->{FILES}} ) {
                        #if ( $tmp->{$mode}->{FILES}->{$package_name}->{SRC} ) {
                        {
                            push @tmp1, $tmp->{$mode}->{FILES}->{$package_name}->{FILENAME};
                        }
                    }
                    print "\t", (join ", ", @tmp1);
                    print "\n";
                }
            }
        }
    }
}
#------------------------------------------------------------------------

sub inquire_for_files_list {
    
    my @to_ajust_mode = ();
    
    # Determine max_mode
    my $max_mode = "p"; 
    foreach my $test_name (@tests_to_run) {
        my $mode = $test_mode{$test_name};
        
        unless ($mode) {
            $mode = "p";
            
            if ( $download_packages ) { 
                if ( $try_beta_packages ) {
                    $mode = "b";
                } else {
                    $mode = "a";
                }
            }
            $test_mode{$test_name} = $mode;
            push @to_ajust_mode, $test_name;
        }

        if ( ($mode eq "a") && ($max_mode eq "p") ) {
            $max_mode = "a"
        } elsif ( ($mode eq "b") && ($max_mode ne "b") ) {
            $max_mode = "b";
        }
    }

    debug_inform ("Max mode: $max_mode");
    list_files ($max_mode); 
    if ( $verbose_lvl >= 2 ) {
        print_files_list ();
    }
    
    unless ( $certification_mode ) {
        # Adjust modes given automaticaly
        foreach my $test_name ( @to_ajust_mode ) {
            if(   ($test_mode{$test_name} eq "b") 
               && !$test_info->{$lsb_version}->{$host_platform}->{$test_name}->{"b"}  ) {
                $test_mode{$test_name} = "a";
                debug_inform("$test_name mode set to 'a'");
            }
            if(   ($test_mode{$test_name} eq "a") 
               && !$test_info->{$lsb_version}->{$host_platform}->{$test_name}->{"a"}  ) {
                $test_mode{$test_name} = "p";
                debug_inform("$test_name mode set to 'p'");
            }
        }
    }
    
    # Check tests availability
    my $i = 0;
    foreach my $test_name (@tests_to_run) {
        if( !$test_info->{$lsb_version}->{$host_platform}->{$test_name}->{$test_mode{$test_name}} ) {
            unless($certification_mode) {
                inform ("Test $test_name is not available with this settings."
                .( ($download_packages && $try_beta_packages)
                        ?"":" You could try to run tests with -D and --beta parameters.")  );
            }
            # Remove it from @tests_to_run
            splice (@tests_to_run, $i, 1);
        }
        $i++;
    }
}
#------------------------------------------------------------------------

# Download and install required packages.
sub prepare_test {
    
    my $test_obj = shift;
    my $test_name = $test_obj->name;
    
    append_string_to_file("$results_dir/host_info/package_versions"
                          , "TEST: $test_name\n");
    
    unless (chdir("$package_dir")) {
        inform ("Could not access $package_dir");
        return 1;
    }
    
    my $mode = $test_mode{$test_name};
    
    my $tmp = $test_info->{$lsb_version}->{$host_platform}->{$test_name}->{$mode};
    
    unless ($tmp) { return 1 };
    
    $run_version{$test_name} = $tmp->{VERSION};
    
    # Download and install required packages
    foreach my $package_name ( split " ", $test_obj->packages )
    {
        my $file = $tmp->{FILES}->{$package_name};
        my $info = package_info($package_name);
        
        debug_inform("$package_name: $info");
        
        # If installed package version doesn't equal to required version
        # or if package not installed ( $info = "" )
        if( $info ne $file->{PKG_VER} ) {
            # Download required files if -D option specified 
            # and file doesn't actually exists in package_dir.
            my $filedest = $file->{SRC}."/".$file->{FILENAME};
            if ( $download_packages && !(-f $package_dir.$filedest) )
            {
                if ( !$file->{SRC} ) {
                    inform ("Can't find package that is considered to be installed: $filedest");
                    return 1;
                }
                # Use wget utility to download files
                my $result = cmd( "$wget -P \"".$package_dir.$file->{SRC}.'" "'
                                  .$FSG_lsb.$filedest.'"' );
                if ( $result != 0 ) {
                    inform ("$test_name: Sorry, wget failed to download $filedest.");
                    return 1;
                }
            }
            
            if ( $info ) {
                remove_package($package_name);
            }
            
            install_package($package_dir.$filedest);
            
            $info = package_info($package_name);
        }
        
        # Check whether the package was installed successfully.
        if ( $info ne $file->{PKG_VER} )
        {
            inform ("Package ".$file->{SRC}."/".$file->{FILENAME}." ($package_name ".$file->{PKG_VER}.") was not correctly installed.");
            return 1;
        }
        
        append_string_to_file("$results_dir/host_info/package_versions"
                              , "  ".$package_name."\t".$info."\n");
    }
    
    foreach my $package_name ( split " ", $test_obj->files )
    {
        my $file = $tmp->{FILES}->{$package_name};
        my $filedest = $file->{SRC}."/".$file->{FILENAME};
        
        if ( $download_packages && !(-f $package_dir.$filedest) )
        {
            if ( !$file->{SRC} ) {
                inform ("Wrong source defined for file: $filedest");
                return 1;
            }
            # Use wget utility to download files
            my $result = cmd( "$wget -P \"".$package_dir.$file->{SRC}.'" "'
                              .$FSG_lsb.$filedest.'"');
            if ( $result != 0 ) {
                inform ("$test_name: wget failed to download $filedest.");
                return 1;
            }
        }
    }

    return 0;
}

1; # return value

# Closes the ftp connection if the execution was interrupted
sub END {
    if ( $ftp ) {
        $ftp->quit;
    }
}
