# LSB Test Execution Framework
# Runtime test module
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Runtime_test;
use strict;

use Common;
use Packages;
use Journal;

use Tests::Test_common;
@Runtime_test::ISA = qw(Test_common);

=head ##
Class Runtime_test Inherits Test_common;
=cut ##

sub new {
    my $self  = Test_common->new();
    $self->{NAME} = "runtime";
    $self->{DESCRIPTION} = "lsb-runtime is a test suite for testing"
        ." core interfaces for conformance to the LSB Specifications.";
    $self->{PACKAGES} .= " lsb-tet3-lite lsb-runtime-test";
    $self->{SOURCES} .= " runtime";
    $self->{BETA_SOURCES} .= " runtime";
    bless($self);
    return $self;
}

sub run {
    my $self = shift;
    
    # Reset the `loop' module in case of absence of loop0 device
    if ( (! -e "/dev/loop0") && does_shell_know("modprobe") ) {
        cmd("modprobe -r loop; modprobe loop");
    }
    
    # Remove the old config file; it would cause 'mv' to ask stupid questions.
    run_command ("rm -f /home/tet/test_sets/TESTROOT/tetexec.cfg.old");
    
    # Short scenario:
    if ( $short_scenarios ) {
        inform("Use short scenario");
        my $result = backup_and_replace( "/home/tet/test_sets/scen.exec",
            "$script_directory/Tests/runtime_short_scen" );
        if ( $result != 0 ) { inform ("Test will be skipped"); return 1; }
    }
    
    return Test_common::run($self);
}

sub journal {
    my $self = shift;
    
    # Parse the journal.
    parse_tet_journal(
        "Certification Tests/Runtime Interface Tests/Runtime Test"
        , "runtime_test",
        "$results_dir/test_journals/runtime_test.journal", $self);
}

sub cleanup {
    my $self = shift;
    
    if ( $short_scenarios ) {
        my $scen_file = "/home/tet/test_sets/scen.exec";
        if (-f "$scen_file.orig" ) {
            cmd ("cp -f $scen_file.orig $scen_file"); 
        }
    }
}

1; # return value