# LSB Test Execution Framework
# Desktop test module
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Desktop_test;
use strict;

use Common;
use Packages;
use Journal;

use Tests::X11::X11_common;
@Desktop_test::ISA = qw(X11_common);

=head ##
Class Desktop_test Inherits X11_common
=cut ##

sub new {
    my $self  = X11_common->new();
    $self->{NAME} = "desktop";
    $self->{DESCRIPTION} = "LSB desktop test suites.";
    $self->{PACKAGES} .= " lsb-tet3-lite lsb-test-desktop";
    $self->{SOURCES} .= " runtime";
    $self->{BETA_SOURCES} .= " runtime";
    bless($self);
    return $self;
}

sub check {
    my $self = shift;
    
    X11_common::check($self);
}

sub run
{
    my $self = shift;
    
    # Short scenario:
    if ( $short_scenarios ) {
        inform("Use short scenario");
        my $result = backup_and_replace( "/opt/lsb/test/desktop/gtkvts/scenarios/func_scen",
            "$script_directory/Tests/X11/desktop_short_scen" );
        if ( $result != 0 ) { inform ("Test will be skipped"); return 1; }
    }
    
    return X11_common::run($self);
}

sub journal
{
    my $self = shift;
    
    parse_tet_journal(
        "Certification Tests/Runtime Interface Tests/Desktop::GTK Test",
        "gtk_test", "$results_dir/test_journals/gtkvts_test.journal", $self);
    parse_tet_journal("Certification Tests/Runtime Interface Tests/Desktop::Libpng Test",
        "libpng_test", "$results_dir/test_journals/libpng_test.journal", $self);
    parse_tet_journal("Certification Tests/Runtime Interface Tests/Desktop::Libxml Test",
        "libxml_test", "$results_dir/test_journals/libxml_test.journal", $self);
    parse_tet_journal("Certification Tests/Runtime Interface Tests/Desktop::Qt3 Test",
        "qt3_test", "$results_dir/test_journals/qt3_test.journal", $self);
    parse_tet_journal("Certification Tests/Runtime Interface Tests/Desktop::Fontconfig Test",
        "fontconfig_test", "$results_dir/test_journals/fontconfig_test.journal", $self);
    if ( $desktop_qt4 ) {
        parse_tet_journal("Certification Tests/Runtime Interface Tests/Desktop::Qt4 Test",
            "qt4_test", "$results_dir/test_journals/qt4_test.journal", $self);
    }
}

sub cleanup {
    my $self = shift;
    
    X11_common::cleanup($self);
    
    if ( $short_scenarios ) {
        my $scen_file = "/opt/lsb/test/desktop/gtkvts/scenarios/func_scen";
        if (-f "$scen_file.orig" ) {
            cmd ("cp -f $scen_file.orig $scen_file"); 
        }
    }
}

1; # return value