#!/usr/bin/perl -w

# LSB Test Execution Framework
# Administration Page Module (tests_admin.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';

sub is_process_finished($) {
	my $PID = $_[0];
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return ($test_line !~ m/^[^Zz]/);
}

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n\n";

print_header("LSB Distribution Testkit Administration", "admin");

print "<h1>LSB Distribution Testkit Administration</h1>\n";

# Building the PID=>PPID and PID=>CMD hashes
my %PPID = ();
my %CMD  = ();
foreach (split(/[\r\n]+/, `ps -A -o pid= -o ppid= -o comm=`)) {
	s/^\s+//g;
	my ($ps_pid, $ps_ppid, $ps_comm) = split(/\s+/, $_, 3);
	$PPID{$ps_pid} = $ps_ppid;
	$CMD{$ps_pid} = $ps_comm;
}

# Getting the most ancient ancestor who is still server.pl process
my $serv_pid = $$;
while (1) {
	if (!$serv_pid) {
		last;
	}
	my $ppid = $PPID{$serv_pid};
	if (($CMD{$serv_pid} =~ /^server.pl(\s.*)?$/) and (($CMD{$ppid} !~ /^server.pl(\s.*)?$/))) {
		last;
	}
	else {
		$serv_pid = $ppid;
	}
}

my $serv_status;
my $serv_stop_button;
if ($serv_pid) {
	$serv_status = "<font color=\"green\">Running</font> (PID: $serv_pid)";
	$serv_stop_button = " onclick=\"javascript:going_to_kill($serv_pid, true);\"";
}
else {
	$serv_status = "<b><font color=\"red\">PID not found!</font></b>";
	$serv_stop_button = " disabled=\"disabled\"";
}

my $test_pid;
my $test_status;
my $test_stop_button;
if (open(FILE, "< /tmp/tests_output.8888")) {
	flock(FILE, LOCK_SH);
	my $waiting_pid = <FILE>;
	flock(FILE, LOCK_UN);
	close(FILE);
	$test_pid = $waiting_pid;
	$waiting_pid =~ s/PID: (\d+),.*/$1/g;
	$waiting_pid =~ s/[\r\n]//g;
	$test_pid =~ s/.*CHLD_PID: (\d+).*/$1/g;
	$test_pid =~ s/[\r\n]//g;
	$test_status = "<font color=\"green\">Running</font> (PID: $test_pid)";
	$test_stop_button = " onclick=\"javascript:going_to_kill($test_pid, false);\"";
}
else {
	$test_status = "<font color=\"red\">Not running</font>";
	$test_stop_button = " disabled=\"disabled\"";
}

print <<DATA;
<table class="main" cellpadding="2" cellspacing="2">
  <tr>
    <td width="100">Server status:</td>
    <td width="150">$serv_status</td>
    <td><input type="button" style="width: 90px;" value="Stop server"$serv_stop_button /></td>
  </tr>
  <tr>
    <td>Tests status:</td>
    <td>$test_status</td>
    <td><input type="button" style="width: 90px;" value="Stop tests"$test_stop_button /></td>
  </tr>
</table>

<script language="javascript" type="text/javascript">
// <![CDATA[
function going_to_kill(pid, is_sevrer) {
	if (confirm("Do you really want to kill the process (PID: " + pid + ")?")) {
		var new_url = "tests_kill.pl?pid=" + pid;
		if (is_sevrer)
			new_url += "&server=1";
		document.location = new_url;
	}
}
// ]]>
</script>
DATA

print_footer();
