#!/usr/bin/perl -w

# LSB Test Execution Framework
# Get Certified Page Module (tests_cert.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my @lsb_versions_list = qw/3.1 3.0/;
my %lsb_profiles_list = ("core" => "Core &amp; C++", "desktop" => "Core &amp; C++ &amp; Desktop");

my %archs_list = ();

my $error_show = " display: none;";
my @error_msg = ();
my @warn_msg = ();

my %profile_data = (
	"name" => "",
	"organization" => "",
	"email" => "",
	"send_email" => "",
	"architecture" => "ia32",
	"keep_packages" => " checked=\"checked\"",
	"lsb_version" => "3.1",
	"lsb_profile" => "desktop",
	"verbose_level" => "1",
	"comments" => ""
);

my $error_text = "";
my $warn_text = "";
my $empty_output = 1;

if (open(FILE, $CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list 2>/tmp/tests_error.$$ |")) {
	while (<FILE>) {
		$empty_output = 0;
		s/[\r\n]//g;
		if (/^ERROR: (.*)$/i) {
			$error_text .= $1."<br />";
		}
		elsif (/^WARNING: (.*)$/i) {
			$warn_text .= $1."<br />";
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^LSB_PROFILE:\s+(.*)$/) {
			$profile_data{"lsb_profile"} = $1;
		}
		elsif (/^(EXPIRED:|FILE_SIZE:|MODULE_TIME:)\s+(.*)$/) {
		}
		elsif (/^[^\s#]/) {
			my ($lsb_version, $arch, $mod, $stage, $vers, $files) = split(/\s+/, $_, 6);
			$archs_list{$arch} = 1;
		}
	}
	close(FILE);
	if (-f "/tmp/tests_error.$$" and (-s "/tmp/tests_error.$$" != 0)) {
		if (!$empty_output) {
			$error_show = "";
			push @error_msg, "There were some errors while retrieving the list of available test modules!<br />See details in the <a href=\"get.pl$APP_DATA/log/server.log.8888\"><u>server log file</u></a>.";
		}
		print STDERR `cat /tmp/tests_error.$$`;
	}
	unlink("/tmp/tests_error.$$");
	if ($empty_output) {
		$error_show = "";
		push @error_msg, "Cannot get the system information: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
	}
	if ($error_text ne "") {
		$error_show = "";
		push @error_msg, $error_text."Please, see the <a href=\"tests_help.pl#troubleshooting\"><u>Troubleshooting</u></a> Help section to find the solution.<br />If you could not find any, please, write at <a href=\"mailto:fsgteam\@ispras.ru\"><u>fsgteam\@ispras.ru</u></a>.";
	}
	if ($warn_text ne "") {
		$error_show = "";
		push @warn_msg, $warn_text."Please, see the <a href=\"tests_help.pl#troubleshooting\"><u>Troubleshooting</u></a> Help section to find the solution.<br />If you could not find any, please, write at <a href=\"mailto:fsgteam\@ispras.ru\"><u>fsgteam\@ispras.ru</u></a>.";
	}
}
else {
	$error_show = "";
	push @error_msg, "Cannot get the system information: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
}

if (!-d "$APP_DATA/profiles") {
	system("mkdir -p $APP_DATA/profiles");
	if (!-d "$APP_DATA/profiles") {
		$error_show = "";
		push @error_msg, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = "";
my $create_session_error = 0;

if (!$_COOKIE{"session_id"} or !-f "$APP_DATA/profiles/~session.$PeerIP.".$_COOKIE{"session_id"}) {
	my $i = time();
	while (-f "$APP_DATA/profiles/~session.$PeerIP.$i") {
		$i++;
	}
	$_COOKIE{"session_id"} = $i;
	$profile_name = "~session.$PeerIP.$i";
	if (open(FILE, "> $APP_DATA/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		$error_show = "";
		push @error_msg, "Cannot create the session profile '$profile_name'<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = "~session.$PeerIP.".$_COOKIE{"session_id"};
}

if (open(FILE, "$APP_DATA/profiles/$profile_name")) {
	my @comments = ();
	while (<FILE>) {
		s/[\r\n]//g;
		if (/^NAME:\s+(.*)$/) {
			$profile_data{"name"} = $1;
		}
		elsif (/^ORGANIZATION:\s+(.*)$/) {
			$profile_data{"organization"} = $1;
		}
		elsif (/^EMAIL:\s+(.*)$/) {
			$profile_data{"email"} = $1;
		}
		elsif (/^SEND_EMAIL:\s+(.*)$/) {
			$profile_data{"send_email"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^VERBOSE_LEVEL:\s+(.*)$/) {
			$profile_data{"verbose_level"} = $1;
		}
		elsif (/^KEEP_PACKAGES:\s+(.*)$/) {
			$profile_data{"keep_packages"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^LSB_PROFILE:\s+(.*)$/) {
			$profile_data{"lsb_profile"} = $1;
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^COMMENTS: (.*)$/) {
			push @comments, $1;
		}
	}
	close(FILE);
	$profile_data{"comments"} = join("\n", @comments);
	if ($profile_data{"email"} eq "") {
		$profile_data{"send_email"} = "";
	}
}
elsif (!$create_session_error) {
	$error_show = "";
	push @error_msg, "Cannot read the session profile data: $profile_name!<br />$!";
}

if ($_COOKIE{"last_error_text"}) {
	$error_show = "";
	push @error_msg, $_COOKIE{"last_error_text"};
}

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n";
print "Set-Cookie: session_id=".$_COOKIE{"session_id"}."; path=/;\n";
print "Set-Cookie: last_error_text=; path=/;\n\n";

print_header("Get Certified", "get_cert");

if ((scalar(@error_msg) != 0) or (scalar(@warn_msg) != 0)) {
	$error_text = "<ul>";
	foreach (@error_msg) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msg) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= "</ul>";
}

$table_spacer = "<td><img src=\"images/environment-spacer.gif\" width=\"5\" height=\"1\" border=\"0\" alt=\"\" /></td>";

print <<DATA;
<noscript><h1 align="center"><font color="red" size="+1">WARNING! Your browser does not support JavaScript.</font></h1></noscript>
<div id="error_msg_collapsed" style="border: dashed 1px darkred; background-color: red; float: right; cursor: pointer; display: none;" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='none';" title="Expand">
<table border="0" cellpadding="1" cellspacing="1">
  <tr>
    <td align="left" style="white-space: nowrap;"><font color="white"><b>&nbsp;There are some problems!</b></font></td>
    <td width="15px"></td>
    <td width="11px"><img src="images/expand.png" alt="O" width="20" height="20" /></td>
  </tr>
</table>
</div>
<div id="error_msg_area" style="border: none;$error_show">
<table border="0" cellpadding="1" cellspacing="1" width="70%" align="center">
  <tr>
    <th style="border: dashed 1px darkred; background: red; color: white; font-size: large;">
      <table border="0" cellpadding="0" cellspacing="0" align="center">
        <tr>
          <td width="100%">Errors!</td>
          <td><img src="images/collapse.png" alt="_" width="20" height="20" style="cursor: pointer;" title="Collapse" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='none';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='';" /></td>
        </tr>
      </table>
    </th>
  </tr>
  <tr><td align="center" style="border: dashed 1px darkred; border-top: none;">
    <table border="0" cellpadding="0" cellspacing="0">
      <tr><td height="10px" colspan="3"></td></tr>
      <tr><td width="50%"></td><td align="left" style="font-size: medium; white-space: nowrap;">$error_text</td><td width="50%"></td></tr>
      <tr><td height="10px" colspan="3"></td></tr>
    </table>
  </td></tr>
</table>
</div>

<div id="tooltip" style="display: none; background-color: #ffffd0; border: solid 1px black; position: absolute;">
<table cellpadding="0" cellspacing="0" border="0" width="200">
  <tr style="background-color: #d0d0d0; color: black;">
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/environment-spacer.gif" width="7" height="1" alt="" /></td>
    <td align="center" width="100%" style="border-bottom: solid 1px black;"><b>Information</b></td>
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/close.png" onclick="javascript:hide_tooltip();" width="20" height="20" title="Close" alt="X" style="cursor: pointer;" /></td>
  </tr>
  <tr>
    <td colspan="3" style="padding: 5px; white-space: nowrap;" id="tooltip_text"></td>
  </tr>
</table>
</div>

<h1>Configure Tests</h1>
<form method="post" action="tests_conf_save.pl" name="configure_test">
<input type="hidden" name="certify" value="1" />
<input type="hidden" name="use_internet" value="on" />
<input type="hidden" name="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="form_action" id="form_action" value="" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td>Your&nbsp;name:</td>
          $table_spacer
          <td colspan="2"><input type="text" name="tester_name" id="tester_name" value="$profile_data{name}" style="width: 150px;" /></td>
        </tr>
        <tr>
          <td>Organization:</td>
          $table_spacer
          <td colspan="2"><input type="text" name="organization" id="organization" value="$profile_data{organization}" style="width: 150px;" /></td>
        </tr>
        <tr>
          <td>E-mail:</td>
          $table_spacer
          <td><input type="text" name="tester_email" value="$profile_data{email}" style="width: 150px;" /></td>
          <td width="100%" style="white-space: nowrap;"><label><input type="checkbox" name="send_email_report" id="send_email_report"$profile_data{send_email} /> Send report via E-mail</label></td>
        </tr>
        <tr>
          <td colspan="4" style="border: dashed 1px gray; background: #ffc000; cursor: pointer;" onclick="javascript:hide_tooltip();show_hide_details();">&nbsp;<img src="images/arr_d.png" alt="" id="block_arrow_img" width="12" height="12" />&nbsp;&nbsp;Additional options</td>
        </tr>
        <tr>
          <td colspan="4" id="additional_data" style="border: dashed 1px gray; border-top: none;">
            <table border="0" cellpadding="2" cellspacing="2" width="100%">
              <tr>
                <td colspan="3"><label><input type="checkbox" name="keep_packages" id="keep_packages"$profile_data{keep_packages} /> Keep packages</label>&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#keep_packages" onclick="javascript:show_hide_tooltip('keep_packages', event); return false;">?</a>]</b></td>
              </tr>
              <tr>
                <td>Verbose&nbsp;Level:</td>
                $table_spacer
                <td width="100%">
                  <select name="verbose_level" style="width: 156px;">
DATA
print "                    <option value=\"1\"".(($profile_data{verbose_level} == 1) ? " selected=\"selected\"" : "").">1 (normal)</option>\n";
print "                    <option value=\"2\"".(($profile_data{verbose_level} == 2) ? " selected=\"selected\"" : "").">2 (high)</option>\n";
print <<DATA;
                  </select>
                </td>
              </tr>
              <tr>
                <td colspan="3">Comments:</td>
              </tr>
              <tr>
                <td colspan="3">
                  <textarea name="comments" rows="5" cols="64">$profile_data{comments}</textarea>
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td colspan="4" valign="bottom" height="15"><hr /></td>
        </tr>
        <tr>
          <td valign="middle" colspan="4">
            <table border="0" cellpadding="0" cellspacing="0" width="100%">
              <tr>
                <td>Architecture:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td>
                  <select name="architecture" id="arch_select" style="width: 80px;">
DATA

if (scalar(keys(%archs_list)) == 0) {
	print "                    <option value=\"$profile_data{architecture}\" selected=\"selected\">$profile_data{architecture}</option>\n";
}
else {
	foreach (sort keys %archs_list) {
		if ($_ eq $profile_data{"architecture"}) {
			print "                    <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			print "                    <option value=\"$_\">$_</option>\n";
		}
	}
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td>LSB&nbsp;Version:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td>
                  <select name="lsb_version" id="lsbv_select" style="width: 80px;" onchange="javascript:control_profile_list();">
DATA

foreach (@lsb_versions_list) {
	if ($_ eq $profile_data{"lsb_version"}) {
		print "                    <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "                    <option value=\"$_\">$_</option>\n";
	}
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td>LSB&nbsp;Profile:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td width="100%">
                  <select name="lsb_profile" id="lsbp_select" style="width: 180px;">
DATA

foreach (sort keys %lsb_profiles_list) {
	if ($_ eq $profile_data{"lsb_profile"}) {
		print "                    <option value=\"$_\" selected=\"selected\">$lsb_profiles_list{$_}</option>\n";
	}
	else {
		print "                    <option value=\"$_\">$lsb_profiles_list{$_}</option>\n";
	}
}

print <<DATA;
                  </select>
                </td>
              </tr>
            </table>
          </td>
        </tr>
      </table>
      <br />
      <table cellpadding="0" cellspacing="0" width="100%" style="border: solid 4px white;"><tr><td>
        &nbsp;<b><a href="" onclick="javascript:var e=document.getElementById('warning');if (e) {e.style.display=(e.style.display=='none'?'':'none');} return false;" title="Click here for details"><font color="red">WARNING!</font> <font color="green">Do not use the tests on any machine holding important data!</font></a></b><br />
        <div id="warning" style="display: none;"><br />
        <div style="border: 1px dashed #2f6fab; background-color: #f9f9f9; width: 90%; padding: 3px; font-family: monospace;">
          WARNING! PLEASE DO NOT USE THIS SCRIPT ON ANY MACHINE THAT HOLDS IMPORTANT DATA! The tests sometimes put a heavy stress on the tested computer, and in rare unfortunate cases they can crash the system or damage the stored data. They also are a security risk as they pass along the root password.
        </div>
        </div>
        <br />
        <input type="submit" name="run_tests" id="run_tests_button" value="Run tests" style="width: 150px;" onclick="javascript:return do_action('run_tests');" /><!--&nbsp;&nbsp; (Estimated&nbsp;execution&nbsp;time:&nbsp;<b><span id="estim_time">0&nbsp;min</span></b>)-->
      </td></tr></table>
    </td>
    <td style="border-right: dashed 1px gray;"><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
DATA

my $profiles_options = "";
my @profiles_list;
if (opendir(DIR, "$APP_DATA/profiles")) {
	@profiles_list = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@profiles_list) {
		$profiles_options .= "              <option value=\"$_\">$_</option>\n";
	}
	closedir(DIR);
}

my $disabled_list = "";
if ($profiles_options eq "") {
	$profiles_options = "              <option value=\"\">&lt;No profiles present&gt;</option>\n";
	$disabled_list = " disabled=\"disabled\"";
}

my $profiles_list_num = scalar(@profiles_list);
if ($profiles_list_num < 2) {
	$profiles_list_num = 2;
}
elsif ($profiles_list_num > 6) {
	$profiles_list_num = 6;
}

print <<DATA;
    <td valign="top" width="50%">
      <h2>User Profile Management&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#profile_management" onclick="javascript:show_hide_tooltip('profile_management', event); return false;">?</a>]</b></h2>
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="3">Select the profile or type its name:</td>
        </tr>
        <tr>
          <td colspan="3">
            <table border="0" cellpadding="0" cellspacing="0"><tr>
              <td style="vertical-align: middle;">
                <input type="text" name="edit_profile_name" id="edit_profile_name" value="" style="width: 200px;" onkeypress="javascript:return onKeyPressEdit(event);" />
              </td>
              <td style="vertical-align: middle;">
                <img src="images/drop_down.png" alt="v" width="16" height="20" id="drop_down_button" style="cursor: pointer;" onclick="javascript:onClickArrow();" />
              </td>
            </tr></table>
            <div id="hidden_div" style="position: absolute; z-index: +1; display: none;">
              <select name="list_profile_name" id="list_profile_name" size="$profiles_list_num" style="width: 222px;"$disabled_list onclick="javascript:onClickList();" onkeypress="javascript:return onKeyPressList(event)">
$profiles_options              </select>
            </div>
          </td>
        </tr>
        <tr>
          <td><input type="submit" name="save_profile_button" id="save_profile_button" value="Save" style="width: 70px;" onclick="javascript:return do_action('save_profile');" /></td>
          <td><input type="submit" name="load_profile_button" id="load_profile_button" value="Load" style="width: 70px;" onclick="javascript:return do_action('load_profile');" /></td>
          <td width="100%"><input type="submit" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 70px;" onclick="javascript:return do_action('delete_profile');" /></td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</form>
<script language="javascript" type="text/javascript">
// <![CDATA[
var preload_image_names = new Array('arr_r.png', 'expand.png', 'collapse.png', 'drop_down_pressed.png', 'close.png');
var preload_images = new Array();
for (var i=0; i<preload_image_names.length; ++i) {
	preload_images[i] = new Image;
	preload_images[i].src = 'images/' + preload_image_names[i];
}

var edit_profile_name = document.getElementById('edit_profile_name');
var list_profile_name = document.getElementById('list_profile_name');
var hidden_div = document.getElementById('hidden_div');
var drop_down_button = document.getElementById('drop_down_button');
var valid_drop_box = list_profile_name && edit_profile_name && hidden_div && drop_down_button;

function show_hide_details() {
	var elem_div = document.getElementById("additional_data");
	var elem_img = document.getElementById("block_arrow_img");
	if (elem_div && elem_img) {
		var to_show = (elem_div.style.display == "none");
		elem_div.style.display = to_show ? "" : "none";
		elem_img.setAttribute("src", to_show ? "images/arr_d.png" : "images/arr_r.png");
	}
}

DATA

my $array_profiles = ((scalar(@profiles_list) > 0) ? "'".join("', '", @profiles_list)."'" : "");

print <<DATA;
function do_action(name) {
	var elem = document.getElementById("form_action");
	if (!elem) {
		alert("Internal error: Cannot find 'form_action' field!\\nPlease, contact the authors.");
		return false;
	}
	if (name == "delete_profile") {
		if (!edit_profile_name){
			alert("Internal error: Cannot find 'edit_profile_name' field!\\nPlease, contact the authors.");
			return false;
		}
		if (edit_profile_name.value) {
			if (!confirm("Do you really want to delete the profile '" + edit_profile_name.value + "'?"))
				return false;
		}
		else {
			alert("Please, specify the profile name!");
			return false;
		}
	}
	else if (name == "load_profile") {
		if (!edit_profile_name){
			alert("Internal error: Cannot find 'edit_profile_name' field!\\nPlease, contact the authors.");
			return false;
		}
		if (!edit_profile_name.value) {
			alert("Please, specify the profile name!");
			return false;
		}
	}
	else if (name == "save_profile") {
		var profile_names = new Array($array_profiles);
		if (!edit_profile_name){
			alert("Internal error: Cannot find 'edit_profile_name' field!\\nPlease, contact the authors.");
			return false;
		}
		if (edit_profile_name.value == "") {
			alert("Please, specify the profile name!");
			return false;
		}
		else {
			for (var i=0; i<profile_names.length; ++i) {
				if (edit_profile_name.value == profile_names[i]) {
					if (!confirm("Do you want to overwrite the profile '" + edit_profile_name.value + "'?"))
						return false;
					break;
				}
			}
		}
	}
	else if (name == "run_tests") {
		var elem1 = document.getElementById("organization");
		var elem2 = document.getElementById("tester_name");
		if (!elem1 || (elem1.value == "") || !elem2 || (elem2.value == "")) {
			alert("Please, specify your name and organization!");
			return false;
		}
	}
	elem.value = name;
	return true;
}

var last_tooltip = '';
var tooltip_texts = new Array();
tooltip_texts['keep_packages'] = 'If this option is turned on, the test packages<br /> remain installed in the system after the test execution.';
tooltip_texts['profile_management'] = 'This section allows you to save all the settings<br />from the left pane into user profiles and load them afterwards.<br />See <a href="tests_help.pl#profile_management" target="_blank" onclick="javascript:hide_tooltip();"><b>Help</b></a> for details.';

var just_shown_tooltip = false;

function show_hide_tooltip(id, event) {
	var tt_window = document.getElementById("tooltip");
	if (tt_window) {
		if (last_tooltip != id) {
			var tt_text = document.getElementById("tooltip_text");
			if (tt_text) {
				tt_text.innerHTML = (tooltip_texts[id] || "<font color=\\"red\\"><b>Sorry, no information for this item was found!</b></font><br />Please, write to <a href=\\"mailto:fsgteam\@ispras.ru\\">fsgteam\@ispras.ru</a>.");
				var ScrollTop = document.body.scrollTop;
				if (ScrollTop == 0)
				{
				    if (window.pageYOffset)
				        ScrollTop = window.pageYOffset;
				    else
				        ScrollTop = (document.body.parentElement) ? document.body.parentElement.scrollTop : 0;
				}
				var ScrollLeft = document.body.scrollLeft;
				if (ScrollLeft == 0)
				{
				    if (window.pageXOffset)
				        ScrollLeft = window.pageXOffset;
				    else
				        ScrollLeft = (document.body.parentElement) ? document.body.parentElement.scrollLeft : 0;
				}
				var div_top = event.clientY + ScrollTop + 10;
				var div_left = event.clientX + ScrollLeft + 10;
				tt_window.style.display = '';
				var div_width = (tt_window.style.pixelWidth || tt_window.offsetWidth);
				var wnd_width = (document.body.clientWidth || window.innerWidth);
				if (div_left + div_width > wnd_width)
					div_left = wnd_width - div_width;
				tt_window.style.top = div_top + 'px';
				tt_window.style.left = div_left + 'px';
				last_tooltip = id;
				just_shown_tooltip = true;
			}
		}
		else {
			tt_window.style.display = 'none';
			last_tooltip = '';
		}
	}
}

function hide_tooltip() {
	var elem = document.getElementById("tooltip");
	if (elem) {
		elem.style.display = 'none';
		last_tooltip = '';
	}
}

var VK_ESC  = 27;
var VK_CR   = 13;
var VK_LF   = 10;
var VK_DOWN = 40;
var VK_TAB  = 9;

var just_shown_list = false;

function show_list() {
	hidden_div.style.display = '';
	drop_down_button.setAttribute("src", "images/drop_down_pressed.png");
	just_shown_list = true;
}

function hide_list(set_focus) {
	if (set_focus)
		edit_profile_name.focus();
	hidden_div.style.display = 'none';
	drop_down_button.setAttribute("src", "images/drop_down.png");
}

function onBodyClick() {
	if (valid_drop_box && (hidden_div.style.display != 'none')) {
		if (!just_shown_list)
			hide_list(false);
		just_shown_list = false;
	}
	if (!just_shown_tooltip)
		hide_tooltip();
	just_shown_tooltip = false;
}

function onKeyPressList(event) {
	if (!valid_drop_box || (hidden_div.style.display == 'none'))
		return true;
	if (event.keyCode == VK_ESC) {
		hide_list(true);
		return false;
	}
	else if ((event.keyCode == VK_CR) || (event.keyCode == VK_LF)) {
		edit_profile_name.value = list_profile_name.value;
		hide_list(true);
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		hide_list(false)
		return true;
	}
	return true;
}

function onClickList() {
	if (!valid_drop_box || (hidden_div.style.display == 'none'))
		return true;
	edit_profile_name.value = list_profile_name.value;
	hide_list(true);
}

function onKeyPressEdit(event) {
	if (!valid_drop_box)
		return true;
	if (event.keyCode == VK_ESC) {
		if (hidden_div.style.display != 'none') {
			hide_list(true);
			return false;
		}
	}
	else if (event.keyCode == VK_DOWN) {
		if (hidden_div.style.display == 'none') {
			show_list();
		}
		list_profile_name.focus();
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		if (hidden_div.style.display != 'none') {
			list_profile_name.focus();
			return false;
		}
	}
	else
		hide_list(true);
	return true;
}

function onClickArrow() {
	if (!valid_drop_box)
		return true;
	if (hidden_div.style.display == 'none') {
		show_list();
		list_profile_name.focus();
	}
	else {
		hide_list(true);
	}
}

function save_and_go(url) {
	if (window.event.shiftKey)
		document.configure_test.target = "_blank";
	document.configure_test.action = "tests_conf_save.pl?goto=" + url.replace("#", "%23");
	var elem = document.getElementById("form_action");
	if (elem) {
		elem.value = name;
		document.configure_test.submit();
		return false;
	}
	else
		return true;
}

function modify_all_links() {
	var links = document.getElementsByTagName("A");
	for (var i = 0; i < links.length; i++) {
		if ((links[i].target == "") && (links[i].href != "") && (links[i].onclick == null)) {
			links[i].onclick = new Function("return save_and_go(\\"" + links[i].href + "\\");");
		}
	}
}

function control_profile_list() {
	var elem_v = document.getElementById("lsbv_select");
	var elem_p = document.getElementById("lsbp_select");
	if (!elem_v || !elem_p)
		return;
	if (elem_v.value == "3.1") {
		elem_p.disabled = false;
	}
	else {
		elem_p.disabled = true;
		elem_p.value = "desktop";
	}
}

control_profile_list();
modify_all_links();
// ]]>
</script>
DATA

print_footer();
