#!/usr/bin/perl -w

# LSB Test Execution Framework
# Help Page Module (tests_help.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n\n";

print_header("Information", "help");

print <<DATA;
<a name="top"></a>
<p align="justify"><b>LSB Distribution Testkit Manager</b> provides a web interface for executing LSB tests.</p><br />
<table style="border: dashed 1px #2f6fab; padding: 5px; white-space: nowrap; background-color: #f9f9f9;"><tr><td>
<h3>Help Contents</h3>
<ol>
  <li><a href="#get_cert">Get&nbsp;Certified page</a></li>
  <li><a href="#conf">Custom&nbsp;Tests page</a></li>
  <li><a href="#exec">Execution page</a></li>
  <li><a href="#results">Results page</a>
    <ul><li><a href="#report">Report page</a></li></ul>
  </li>
  <li><a href="#help">Help page</a></li>
  <li><a href="#about">About page</a></li>
  <li><a href="#admin">Administration page</a></li>
  <li><a href="#troubleshooting">Troubleshooting</a></li>
</ol></td></tr>
</table>
<br />

<a name="get_cert"></a><h2>Get&nbsp;Certified</h2>
<p align="justify">
Open this page to start normal LSB certification. This page is very similar to the <a href="#conf"><b>Custom&nbsp;Tests</b></a> page, but has some restrictions implied by the certification process:
</p>
<ul>
<li>The <b>Your name</b> and <b>Organization</b> fields are mandatory.</li>
<li>It is impossible to turn off using Internet: all the test packages must be downloaded by the LSB Distribution Testkit if they are not available locally.</li>
<li>It is impossible to use short scenarios.</li>
<li>The test selection list is turned off (the most recent versions of all the necessary tests are executed).</li>
<li>The additional option <b>LSB&nbsp;Profile</b> allows to test against different LSB profiles (it is not needed for the <a href="#conf"><b>Custom&nbsp;Tests</b></a> page as there are more flexible settings available).</li>
</ul>
<p align="justify">
After the tests execution is finished the browser is automatically redirected to the special results page, which represents a simplified variant of the tests execution report. This page also describes the steps on getting the certification.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="conf"></a><h2>Custom&nbsp;Tests</h2>
<p align="justify">
This page allows you to configure the test system. The following options are present:
</p>
<ul>
<li><b>Your name:</b><br />The name of the user who started the tests. This field is reserved for future use (currently not used).</li>
<li><b>Organization:</b><br />The name of your organization. This field is reserved for future use (currently not used).</li>
<li><b>E-mail:</b><br />Specify your E-mail here. It will be used for sending the report after the tests finish.</li>
<li><b>Send report via E-mail</b><br />Choose whether to send the report via E-mail using the specified address.</li>
<li><b>Additional options</b><br />Less important additional settings are shown here. To show/hide them just click the orange header. This block contains the following settings:
  <ul>
  <li><a name="keep_packages"></a><b>Keep packages</b><br />Don't remove packages that were installed to execute the tests. If this option is turned off, the packages are deinstalled after the tests are finished. Keeping the packages allows the tests to start next time without reinstalling, which saves time.<br /><font color="blue"><b>Warning:</b></font> If you set the <b>Keep packages</b> option, please do not run tests for different architectures. The Framework cannot determine which architecture the installed package is designed for, and could run the wrong set of tests. For example, you may want to run tests for the <b>x86_64</b> platform, but start the tests for the <b>ia32</b> platform instead, because those tests were already installed.</li>
  <li><a name="short_scenarios"></a><b>Short scenarios</b><br />This option can set up runing a reduced set of tests, which can take less time. It is present only in test versions of the LSB Distribution Testkit and designed to simplify the process of reproducing errors when providing feedback.</li>
  <li><b>Verbose Level:</b><br />Specifies how much information should be given by the tests to the output console.</li>
  <li><b>Comments:</b><br />Here you may write comments with any additional information. These comments are stored in the execution report and shown in the <a href="#results"><b>Results</b></a> page.</li>
  </ul>
</li>
<li><b>Use Internet for downloading the tests</b><br />This checkbox allows you to specify not to use any Internet connections for downloading the test packages. If it is turned off, you will not be able to refresh the list of the tests.</li>
<li><b>Architecture:</b><br />Select the machine architecture here. NOTE: Only architectures that are compatible with the machine being tested are listed.</li>
<li><b>LSB Version:</b><br />This combobox specifies the LSB version to check.</li>
<li><b>Refresh list</b><br />This button causes the Testkit to connect to the <a href="ftp://linux-foundation.org/" target="_blank">LF's FTP server</a>, obtain the list of the latest test packages, and update the list of packages shown below based on downloaded and newly available packages.</li>
</ul>
<p align="justify">
Below is the list of available test packages present. At first start it will most likely be empty. To get the full list press the <b>Refresh list</b> button (if the list is empty this button changes its name to <b>Get List</b>). It may take about 20 to 40 seconds depending on your Internet connection speed and load of the servers. The list from FTP is cached, so you don't need to refresh it every time. If the list has not been updated for some time (more than 24 hours), the LSB Distribution Testkit shows you a warning that the list may be out-of-date.<br />
The list of test suites consists of four columns. The first column contains the name of the test suite and a checkbox indicating whether to execute this test or not (if it was set and then disabled, e.g. when you turned off the <b>Use Internet connection</b> checkbox, the test will not be executed). The other three columns allow you to choose which version of the test suite to execute. These columns are:
</p>
<ol>
<li><b>Localhost</b><br />The version of the test suite that is present on your local hard disk (either downloaded into the <font color="blue">$APP_DATA/lsb-packages</font> directory or one of its subdirectories, or already installed).</li>
<li><b>FTP</b><br />The latest <b>stable</b> version present on the LF's FTP server.</li>
<li><b>FTP (beta)</b><br />The latest <b>beta</b> version present on the LF's FTP server.</li>
</ol>
<p align="justify">
If some of the items do not contain any version number, it means that the corresponding version is not available.
</p>
<p align="justify">
All the test suites are divided into the following groups and subgroups.
</p>
<ul><li><a name="cert"></a><b>[Certification Tests]</b>: Test suites that are used for certification of distributions for conformance to the LSB Specifications.
        <ul><li><a name="cmdtests"></a><b>[Command Tests]</b>: Test suites that verify the commands required by LSB specification.
                <ul><li><a name="cmdchk"></a><font color="green">Command Check (lsb-cmdchk)</font>: This test suite verifies that all the commands required by the LSB specification are present. It does not check the behaviour of those commands.</li></ul>
            </li>
            <li><a name="sitests"></a><b>[Static Interface Tests]</b>: Test suites that verify interfaces in the runtime libraries.
                <ul><li><a name="libchk"></a><font color="green">Library Check (lsb-libchk)</font>: This test suite is used to check if the runtime libraries contain all of the interfaces with correct versions as specified by the LSB specification.</li></ul>
            </li>
            <li><a name="ritests"></a><b>[Runtime Interface Tests]</b>: Test suites for testing LSB runtime environments for conformance to the LSB Specifications.
                <ul><li><a name="desktop"></a><font color="green">Desktop Tests</font> consist of the following test suites:
                        <ul><li><font color="green">GTK Test</font></li>
                            <li><font color="green">Libpng Test</font></li>
                            <li><font color="green">Libxml Test</font></li>
                            <li><font color="green">Qt3 Test</font></li>
                            <li><font color="green">Fontconfig Test</font></li>
                        </ul>
                    </li>
                    <li><a name="libstdcpp"></a><font color="green">Libstdc++ Test</font>: This is the libstdc++-v3 testsuite from gcc version 3.4.3, modified to run against an installed version of libstdc++ without requiring a compiler.</li>
                    <li><a name="runtime"></a><font color="green">Runtime Test</font>: This is a test suite for testing core interfaces for conformance to the LSB Specifications.</li>
                    <li><a name="vsw4"></a><font color="green">X11 Test (vsw4)</font>: LSB X Window System test suite. These tests are derived from the VSW4 XTest test suites.</li>
                </ul>
            </li>
            <li><a name="abtests"></a><b>[Application Battery Tests]</b>: Tests that verify the correct operation of binary applications from the LSB application battery.
                <ul><li><a name="apache"></a><font color="green">Apache</font>: versatile, high-performance HTTP server.</li>
                    <li><a name="expect"></a><font color="green">Expect</font>: a program that 'talks' to other programs.</li>
                    <li><a name="groff"></a><font color="green">Groff</font>: a document formatting system.</li>
                    <li><a name="lynx"></a><font color="green">Lynx</font>: a text-based Web browser.</li>
                    <li><a name="python"></a><font color="green">Python</font>: an interpreted, interactive, object-oriented programing language.</li>
                    <li><a name="raptor"></a><font color="green">Raptor</font>: parsers and serializers for RDF.</li>
                    <li><a name="rsync"></a><font color="green">Rsync</font>: a program for synchronizing files over a network.</li>
                    <li><a name="samba"></a><font color="green">Samba</font>: a file and print server for SMB/CIFS clients.</li>
                    <li><a name="tcl"></a><font color="green">Tcl</font>: an embedded scripting language.</li>
                    <li><a name="xpdf"></a><font color="green">Xpdf</font>: a PDF file viewer for X Window System.</li>
                </ul>
            </li>
        </ul>
    </li>
    <li><a name="unoff"></a><b>[Unofficial Tests]</b>: Tests that are not used for certification, but can be used for futher LSB compliance testing.
        <ul><li><a name="uritests"></a><b>[Runtime Interface Tests]</b>: Test suites for testing LSB runtime environments for conformance to the LSB Specifications.
                <ul><li><a name="xts5"></a><font color="green">Xts5 Test (xts5)</font>: This is a binary version of the LSB X Window System test suite. These tests are derived from the XTS5 test suites.</li></ul>
            </li>
        </ul>
    </li>
</ul>
<p align="justify">
Some of the tests may have additional options. Currently these are <b>Command Check</b>, <b>Library Check</b> and <b>Desktop</b> tests.
</p>
<p align="justify">
When you select the tests, you can see the approximate summary size of the packages to be downloaded (in the <b>Download size:</b> field just above the list of the tests). If you turn the option <b>Use Internet connection for downloading the tests</b> off, all test packages which need downloading will be automatically disabled.
</p>
<p align="justify">
After you selected the test packages to run you may press the button <b>Run selected tests</b> located at the bottom of the page. This will redirect you immediately to the next page (<a href="#exec"><b>Execution</b></a>) and the tests will start. If the tests are already (or still) running you will get the error message at the top of the page, because it is currently impossible to run several tests instances simultaneously.
</p>

<a name="profile_management"></a><h3>User Profile Management</h3>
<p align="justify">
This section allows you to save/load all the settings listed above in different user profile files. To save the settings just specify the new profile name in the edit box (or use the drop-down list to select the existing profile and overwrite it), and then press the <b>Save</b> button. To load a previously saved profile, select it from the drop-down list and press <b>Load</b>. The <b>Delete</b> button allows you to delete the old profiles that are no longer needed.<br />
<b>Note:</b> The profile is not automatically saved when you run the tests! The settings are just stored in a temporary <i>session profile</i> which is automatically removed when it is no longer used. So if you wish to use the same set of options later, please save it explicitly, via the <b>Save</b> button.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="exec"></a><h2>Execution</h2>
<p align="justify">
This page allows you to watch the tests execution progress. If the tests are already running, you may open this page to watch the progress (even from another computer). If the tests are not running at the moment, you may select the user profile that was saved earlier on the <a href="#conf"><b>Custom&nbsp;Tests</b></a> page and run the tests by pressing the <b>Start</b> button. The <b>Stop</b> button allows you to forcibly stop the tests execution.
</p>
<p align="justify">
Just below the current status of the tests execution and its output console are shown. The console is refreshed automatically about every second and displays all output for the tests (both <b>stdout</b> and <b>stderr</b>). If the tests have finished, the status may contain any error codes returned. If the tests were stopped, the received signal is shown.
</p>
<p align="justify">
If the report has been created, the link <b>View test report</b> is shown below the output console. Just click it to view this report.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="results"></a><h2>Results</h2>
<p align="justify">
This page shows the table with the full list of the tests execution results. The table contains the following columns:
</p>
<ul>
<li><b>Date/Time</b><br />Here the date and time are shown, when the tests were finished. At the left edge of this column a status-icon is shown:
<ul>
<li><img src="images/CertPassed.png" alt="CertPass" width="33" height="30" /> The certification tests passed.</li>
<li><img src="images/CertFailed.png" alt="CertFail" width="33" height="30" /> The certification tests failed.</li>
<li><img src="images/TestPassed.png" alt="Pass" width="33" height="30" /> The custom set of tests passed.</li>
<li><img src="images/TestFailed.png" alt="Fail" width="33" height="30" /> The custom set of tests failed.</li>
</ul>
</li>
<li><b>Summary</b><br />This column shows the general results of the tests (how many tests passed and failed).</li>
<li><b>Comment</b><br />Here the comments entered on the <a href="#conf"><b>Custom&nbsp;Tests</b></a> page are shown.</li>
</ul>
<p align="justify">
To view the detailed particular report just click the link in the <b>Date/Time</b> field (see the <a href="#report"><b>Report page</b></a> section below for details).
</p>
<p align="justify">
To remove the reports that are no longer needed, just mark them with the checkboxes shown at the left edge of the corresponding lines and press the <b>Remove selected</b> button below the table.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="report"></a><h2>Report page</h2>
<p align="justify">
This page contains the detailed report about the test results. It consists of the following sections:
</p>
<ul>
<li><b>Test info</b><br />
Contains the information about the machine where the tests were executed.</li>
<li><b>Tests executed</b><br />
An index with links to details and a short summary for each test.<br />
<a name="No_result"></a>If <b>No result</b> is shown, this means that the test failed to run or was interrupted during the execution.
</li>
<li><b>Details for each test</b><br />
    Here the information from the test journal is shown, including the list of warnings and errors detected. You may click any line with particular problem to see more details about this problem. The <b>Severity</b> column contains the status of the problem. Here are the descriptions for some of the status messages:
    <ul>
        <li><b>Warning</b> means that this some non-critical problem occured.</li>
        <li><b>Failed</b> means that a severe problem occured (it usually implies that the distribution is not LSB compliant).</li>
        <li><a name="FIP"></a><b>FIP</b> stands for "<i>Futher Information Provided</i>". These FIP results occur when the test suite is unable to determine whether a result is a PASS or a FAIL. For LSB Certification you have to resolve each FIP case and upload your comments (in plain text) along with the test journals.</li>
        <li><b>Quelled</b> means that the problem is waived and will not prevent the distro from being LSB certified.</li>
    </ul>
</li>
</ul>
<p align="right"><a href="#top">Back to top</a></p>

<a name="help"></a><h2>Help</h2>
<p align="justify">
Contains the description of the web-interface of LSB Distribution Testkit (the page you are reading now).
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="about"></a><h2>About</h2>
<p align="justify">
Contains the general information about LSB Distribution Testkit: its version, license etc.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="admin"></a><h2>Administration</h2>
<p align="justify">
This page is designed for performing administrative tasks. Please be careful doing anything on this page!
</p>
<p align="justify">
Currently the following items are present:
</p>
<ul>
<li><b>Server status:</b><br />Shows current status of the web-server and its PID. The button <b>Stop server</b> shuts the server down. Please note that you will not be able to use the web interface after that!</li>
<li><b>Tests status:</b><br />Shows current status of the tests and PID of their parent process. The button <b>Stop tests</b> forcibly stops execution of the tests (it has the same meaning as the <b>Stop</b> button on the <a href="#exec"><b>Execution</b></a> page).</li>
</ul>
<p align="right"><a href="#top">Back to top</a></p>

<a name="troubleshooting"></a><h2>Troubleshooting</h2>
<ul>
<li>Some locales are required for libstdc++ test. That may turn into a problem for your distro. If that's the case, maybe the following solution for Debian will help you:<br />
<a href="http://people.debian.org/~taggart/lsb/tests.html" target="_blank">http://people.debian.org/~taggart/lsb/tests.html</a>
</li>

<li>In the case when the autotest couldn't find fonts, there is a workaround. First, find the font directory (see the files <font color="blue"><code>/etc/X11/xorg.conf</code></font> and <font color="blue"><code>/etc/X11/fs/config</code></font>). Also you may try the command<br />
<code>find / -name 75dpi</code><br />
to find all 75dpi font directories. Choose the appropriate font directory and make sure that the following subdirectories are present: 'misc', '75dpi', '100dpi'.<br />
Autotest tries to find these subdirectories in the following locations:<br />
<ul>
<li><code>/usr/X11R6/lib/X11/fonts/</code></li>
<li><code>/usr/share/fonts/X11/</code></li>
<li><code>/usr/share/X11/fonts/</code></li>
<li><code>/usr/share/fonts/</code></li>
</ul>
Create symlinks to the found directories (misc, 75dpi, 100dpi) into one of the directories listed above so that autotest could find them next time.
</li>

<li>If you have no 100dpi fonts installed, you should install them using the tools provided by your distro.</li>
</ul>

<p align="justify">
Here are some of the possible errors and the ways of solving them.
</p>
<ul>
<li><b>Error: Couldn't recognize the host platform.</b><br />
If you believe that your platform is one of the listed supported platforms, but its name wasn't recognized by the Test Execution Framework (TEF), please first of all send the bugreport (see contacts below). If you use the command-line tool, you can just specify the platform using the -p parameter, e.g.:<br />
<code>./lsb-tef.pl -p ppc32</code><br />
to state the platform is ppc32. Unfortunately, there is no way to specify the platform with the web UI.<br />
(You can identify your platform using the command <code>uname -i</code>.)
</li>

<li><b>Error: Couldn't recognize package manager for this distro.</b><br />
or <b>Error: ... Unknown package manager ...</b><br />
If you are sure that your package manager is <b>RPM</b> or <b>dpkg</b> with <b>alien</b> installed, there is a method to state the correct package manager. If you use the command-line tool, you can just specify the package manager using the -pm option, e.g.<br />
for RPM: <code>./lsb-tef.pl -pm RPM</code><br />
for dpkg: <code>./lsb-tef.pl -pm dpkg</code><br />
Unfortunately, there is no way to specify the package manager with the webUI.
</li>

<li><b>Error: Couldn't find LSB loader (loader_name) for this platform (host_platform).</b><br />
or <b>Error: No LSB modules are available.</b><br />
Most likely, you haven't installed the LSB support modules. You should install them using the tools provided by your distribution. (For example, <code>apt-get install lsb</code> should work on Debian-based distros.)
</li>

<li><b>Error: Couldn't find a required tool: '&lt;some tool&gt;'</b><br />
This error message occurs when you are missing a tool used by TEF. Please install the tool in the usual way for your distribution.
</li>

<li><b>Error: Can't find lsb-autotest in &lt;some_dir&gt;.</b><br />
'lsb_autotest' is a tool used by TEF to automatically run the tests. TEF tries to find lsb_autotest at the specified location (usually at manager/autotest). Check that the file manager/autotest/lsb-autotest exists and has appropriate rights to be executed by root (TEF runs tests as root). If the file doesn't exists, it is most likely that TEF wasn't installed correctly; reinstalling it should fix the problem.
</li>

<li><b>Error: Could not open problem database &lt;file_name&gt;</b><br />
'problem_db' is a file that contains information about some known problems. Information in this file is used for the report creation. This file should be located at manager/autotest-ext/problem_db. Check that this file exists and has appropriate privileges to be read by root (TEF works as root). If the file doesn't exists, it is most likely that TEF wasn't installed correctly; reinstalling it should fix the problem.
</li>

<li><b>Error: FTP init failed.</b><br />
or <b>Error: FTP login (anonymous) failed.</b><br />
This usually indicates a poor Internet connection or high load of the LF's FTP server. Try again later.<br />
If you have all the test packages downloaded (all tests are available at the first column <b>localhost</b> on the <a href="#conf"><b>Custom&nbsp;Tests</b></a> page of webUI), you can choose not to download the newer packages, and TEF will not connect to the FTP server.
</li>

<li><b>Error: Failed to execute: &lt;command&gt;</b><br />
or <b>Error: Couldn't open file '&lt;file_name&gt;' for writing.</b><br />
or <b>Error: Failed to copy: ...</b><br />
This means that the TEF failed to fulfil some service operation. Your bugreport with the log of console output attached will be truly appreciated.
</li>
</ul>

<p align="justify">
If you can't find a suitable solution, you can write to <a href="mailto:fsgteam\@ispras.ru">fsgteam\@ispras.ru</a>.
</p>
<p align="right"><a href="#top">Back to top</a></p>
DATA

print_footer();
